# -*- coding: utf-8 -*-
#
#  setup.py
#  jp-reading-alt
# 
#  Created by Lars Yencken on 10-04-2009.
#  Copyright 2009 Lars Yencken. All rights reserved.
#

"""
Package setup file for the jp-reading-alt package.
"""

#----------------------------------------------------------------------------#

from setuptools import setup
import os
import re

#----------------------------------------------------------------------------#

VERSION = '0.1.1'

f = open('src/__version__.py', 'w')
f.write('# Autogenerated by setup.py\n')
f.write('version = "%s"\n' % VERSION)
f.close()

setup(
        name='jp-reading-alt',
        description="Probabilistic reading error models for Japanese kanji.",
        long_description = """
        Reading alternation models for Japanese kanji, considering effects of:
        sequential voicing, sound euphony, vowel length and palatalization
        errors.
        """,
        url="http://bitbucket.org/lars512/jp-reading-alt/",
        version=VERSION,
        author="Lars Yencken",
        author_email="lljy@csse.unimelb.edu.au",
        license="BSD",
        install_requires=[
                'cjktools',
                'cjktools_data',
                'simplestats',
                'django',
                'django-hierarchy',
                'south',
            ],
        package_dir={'jp_reading_alt': 'src'},
        packages=['jp_reading_alt'],
    )
